@@ -18,20 +18,28 @@ import uuid
 import warnings
 from collections import OrderedDict
 
+from flask import current_app, request, url_for
 from werkzeug.utils import import_string
-from flask import request, current_app, url_for
 
-from ._compat import PY2, iteritems
+from flask_caching._compat import PY2, iteritems
 
-__version__ = '1.4.0'
+__version__ = "1.6.0"
 
 logger = logging.getLogger(__name__)
 
-TEMPLATE_FRAGMENT_KEY_TEMPLATE = '_template_fragment_cache_%s%s'
+TEMPLATE_FRAGMENT_KEY_TEMPLATE = "_template_fragment_cache_%s%s"
+SUPPORTED_HASH_FUNCTIONS = [
+    hashlib.sha1,
+    hashlib.sha224,
+    hashlib.sha256,
+    hashlib.sha384,
+    hashlib.sha512,
+    hashlib.md5,
+]
 
 # Used to remove control characters and whitespace from cache keys.
-valid_chars = set(string.ascii_letters + string.digits + '_.')
-delchars = ''.join(c for c in map(chr, range(256)) if c not in valid_chars)
+valid_chars = set(string.ascii_letters + string.digits + "_.")
+delchars = "".join(c for c in map(chr, range(256)) if c not in valid_chars)
 if PY2:
     null_control = (None, delchars)
 else:
@@ -47,8 +55,11 @@ def get_arg_names(f):
     try:
         # Python >= 3.3
         sig = inspect.signature(f)
-        return [parameter.name for parameter in sig.parameters.values()
-                if parameter.kind == parameter.POSITIONAL_OR_KEYWORD]
+        return [
+            parameter.name
+            for parameter in sig.parameters.values()
+            if parameter.kind == parameter.POSITIONAL_OR_KEYWORD
+        ]
     except AttributeError:
         try:
             # Python >= 3.0
@@ -83,48 +94,50 @@ def function_namespace(f, args=None):
 
     instance_token = None
 
-    instance_self = getattr(f, '__self__', None)
+    instance_self = getattr(f, "__self__", None)
 
     if instance_self and not inspect.isclass(instance_self):
         instance_token = repr(f.__self__)
-    elif m_args and m_args[0] == 'self' and args:
+    elif m_args and m_args[0] == "self" and args:
         instance_token = repr(args[0])
 
     module = f.__module__
 
-    if m_args and m_args[0] == 'cls' and not inspect.isclass(args[0]):
-        raise ValueError('When using `delete_memoized` on a '
-                         '`@classmethod` you must provide the '
-                         'class as the first argument')
+    if m_args and m_args[0] == "cls" and not inspect.isclass(args[0]):
+        raise ValueError(
+            "When using `delete_memoized` on a "
+            "`@classmethod` you must provide the "
+            "class as the first argument"
+        )
 
-    if hasattr(f, '__qualname__'):
+    if hasattr(f, "__qualname__"):
         name = f.__qualname__
     else:
-        klass = getattr(f, '__self__', None)
+        klass = getattr(f, "__self__", None)
 
         if klass and not inspect.isclass(klass):
             klass = klass.__class__
 
         if not klass:
-            klass = getattr(f, 'im_class', None)
+            klass = getattr(f, "im_class", None)
 
         if not klass:
             if m_args and args:
-                if m_args[0] == 'self':
+                if m_args[0] == "self":
                     klass = args[0].__class__
-                elif m_args[0] == 'cls':
+                elif m_args[0] == "cls":
                     klass = args[0]
 
         if klass:
-            name = klass.__name__ + '.' + f.__name__
+            name = klass.__name__ + "." + f.__name__
         else:
             name = f.__name__
 
-    ns = '.'.join((module, name))
+    ns = ".".join((module, name))
     ns = ns.translate(*null_control)
 
     if instance_token:
-        ins = '.'.join((module, name, instance_token))
+        ins = ".".join((module, name, instance_token))
         ins = ins.translate(*null_control)
     else:
         ins = None
@@ -139,9 +152,6 @@ def make_template_fragment_key(fragment_name, vary_on=[]):
     return TEMPLATE_FRAGMENT_KEY_TEMPLATE % (fragment_name, "_".join(vary_on))
 
 
-#: Cache Object
-################
-
 class Cache(object):
     """This class is used to control the cache objects."""
 
@@ -173,20 +183,25 @@ class Cache(object):
             base_config.update(config)
         config = base_config
 
-        config.setdefault('CACHE_DEFAULT_TIMEOUT', 300)
-        config.setdefault('CACHE_THRESHOLD', 500)
-        config.setdefault('CACHE_KEY_PREFIX', 'flask_cache_')
-        config.setdefault('CACHE_MEMCACHED_SERVERS', None)
-        config.setdefault('CACHE_DIR', None)
-        config.setdefault('CACHE_OPTIONS', None)
-        config.setdefault('CACHE_ARGS', [])
-        config.setdefault('CACHE_TYPE', 'null')
-        config.setdefault('CACHE_NO_NULL_WARNING', False)
+        config.setdefault("CACHE_DEFAULT_TIMEOUT", 300)
+        config.setdefault("CACHE_IGNORE_ERRORS", False)
+        config.setdefault("CACHE_THRESHOLD", 500)
+        config.setdefault("CACHE_KEY_PREFIX", "flask_cache_")
+        config.setdefault("CACHE_MEMCACHED_SERVERS", None)
+        config.setdefault("CACHE_DIR", None)
+        config.setdefault("CACHE_OPTIONS", None)
+        config.setdefault("CACHE_ARGS", [])
+        config.setdefault("CACHE_TYPE", "null")
+        config.setdefault("CACHE_NO_NULL_WARNING", False)
 
-        if config['CACHE_TYPE'] == 'null' and \
-                not config['CACHE_NO_NULL_WARNING']:
-            warnings.warn("Flask-Cache: CACHE_TYPE is set to null, "
-                          "caching is effectively disabled.")
+        if (
+            config["CACHE_TYPE"] == "null"
+            and not config["CACHE_NO_NULL_WARNING"]
+        ):
+            warnings.warn(
+                "Flask-Caching: CACHE_TYPE is set to null, "
+                "caching is effectively disabled."
+            )
 
         if self.with_jinja2_ext:
             from .jinja2ext import CacheExtension, JINJA_CACHE_ATTR_NAME
@@ -197,36 +212,38 @@ class Cache(object):
         self._set_cache(app, config)
 
     def _set_cache(self, app, config):
-        import_me = config['CACHE_TYPE']
-        if '.' not in import_me:
+        import_me = config["CACHE_TYPE"]
+        if "." not in import_me:
             from . import backends
 
             try:
                 cache_obj = getattr(backends, import_me)
             except AttributeError:
-                raise ImportError("%s is not a valid FlaskCache backend" % (
-                                  import_me))
+                raise ImportError(
+                    "%s is not a valid Flask-Caching backend" % (import_me)
+                )
         else:
             cache_obj = import_string(import_me)
 
-        cache_args = config['CACHE_ARGS'][:]
-        cache_options = {'default_timeout': config['CACHE_DEFAULT_TIMEOUT']}
+        cache_args = config["CACHE_ARGS"][:]
+        cache_options = {"default_timeout": config["CACHE_DEFAULT_TIMEOUT"]}
 
-        if config['CACHE_OPTIONS']:
-            cache_options.update(config['CACHE_OPTIONS'])
+        if config["CACHE_OPTIONS"]:
+            cache_options.update(config["CACHE_OPTIONS"])
 
-        if not hasattr(app, 'extensions'):
+        if not hasattr(app, "extensions"):
             app.extensions = {}
 
-        app.extensions.setdefault('cache', {})
-        app.extensions['cache'][self] = cache_obj(app, config, cache_args,
-                                                  cache_options)
+        app.extensions.setdefault("cache", {})
+        app.extensions["cache"][self] = cache_obj(
+            app, config, cache_args, cache_options
+        )
         self.app = app
 
     @property
     def cache(self):
         app = current_app or self.app
-        return app.extensions['cache'][self]
+        return app.extensions["cache"][self]
 
     def get(self, *args, **kwargs):
         """Proxy function for internal cache object."""
@@ -264,8 +281,15 @@ class Cache(object):
         """Proxy function for internal cache object."""
         return self.cache.get_dict(*args, **kwargs)
 
-    def cached(self, timeout=None, key_prefix='view/%s', unless=None,
-               forced_update=None, query_string=False):
+    def cached(
+        self,
+        timeout=None,
+        key_prefix="view/%s",
+        unless=None,
+        forced_update=None,
+        query_string=False,
+        hash_method=hashlib.md5,
+    ):
         """Decorator. Use this to cache a function. By default the cache key
         is `view/request.path`. You are able to use this decorator with any
         function by changing the `key_prefix`. If the token `%s` is located
@@ -336,7 +360,11 @@ class Cache(object):
                              were passed in a different order. See
                              _make_cache_key_query_string() for more
                              details.
+        :param hash_method: Default hashlib.md5. The hash method used to
+                            generate the keys for cached results.
+
         """
+
         def decorator(f):
             @functools.wraps(f)
             def decorated_function(*args, **kwargs):
@@ -348,8 +376,9 @@ class Cache(object):
                     if query_string:
                         cache_key = _make_cache_key_query_string()
                     else:
-                        cache_key = _make_cache_key(args, kwargs,
-                                                    use_request=True)
+                        cache_key = _make_cache_key(
+                            args, kwargs, use_request=True
+                        )
 
                     if callable(forced_update) and forced_update() is True:
                         rv = None
@@ -358,30 +387,38 @@ class Cache(object):
                 except Exception:
                     if self.app.debug:
                         raise
-                    logger.exception("Exception possibly due to "
-                                     "cache backend.")
+                    logger.exception(
+                        "Exception possibly due to " "cache backend."
+                    )
                     return f(*args, **kwargs)
 
                 if rv is None:
                     rv = f(*args, **kwargs)
                     try:
                         self.cache.set(
-                            cache_key, rv,
-                            timeout=decorated_function.cache_timeout
+                            cache_key,
+                            rv,
+                            timeout=decorated_function.cache_timeout,
                         )
                     except Exception:
                         if self.app.debug:
                             raise
-                        logger.exception("Exception possibly due to "
-                                         "cache backend.")
+                        logger.exception(
+                            "Exception possibly due to " "cache backend."
+                        )
                 return rv
 
             def make_cache_key(*args, **kwargs):
                 # Convert non-keyword arguments (which is the way
                 # `make_cache_key` expects them) to keyword arguments
                 # (the way `url_for` expects them)
-                argspec = inspect.getargspec(f)
-                for arg_name, arg in zip(argspec.args, args):
+                try:
+                    # Python >= 3.0
+                    argspec_args = inspect.getfullargspec(f).args
+                except AttributeError:
+                    argspec_args = inspect.getargspec(f).args
+
+                for arg_name, arg in zip(argspec_args, args):
                     kwargs[arg_name] = arg
 
                 return _make_cache_key(args, kwargs, use_request=False)
@@ -401,22 +438,20 @@ class Cache(object):
                 # are the same, regardless of the order in which they are
                 # provided.
                 args_as_sorted_tuple = tuple(
-                    sorted(
-                        (pair for pair in request.args.items(multi=True))
-                    )
+                    sorted((pair for pair in request.args.items(multi=True)))
                 )
                 # ... now hash the sorted (key, value) tuple so it can be
-                # used as a key for cache. Turn them into bytes so that md5
-                # will accept them
+                # used as a key for cache. Turn them into bytes so that the
+                # hash function will accept them
                 args_as_bytes = str(args_as_sorted_tuple).encode()
-                hashed_args = str(hashlib.md5(args_as_bytes).hexdigest())
+                hashed_args = str(hash_method(args_as_bytes).hexdigest())
                 cache_key = request.path + hashed_args
                 return cache_key
 
             def _make_cache_key(args, kwargs, use_request):
                 if callable(key_prefix):
                     cache_key = key_prefix()
-                elif '%s' in key_prefix:
+                elif "%s" in key_prefix:
                     if use_request:
                         cache_key = key_prefix % request.path
                     else:
@@ -431,16 +466,24 @@ class Cache(object):
             decorated_function.make_cache_key = make_cache_key
 
             return decorated_function
+
         return decorator
 
     def _memvname(self, funcname):
-        return funcname + '_memver'
+        return funcname + "_memver"
 
     def _memoize_make_version_hash(self):
-        return base64.b64encode(uuid.uuid4().bytes)[:6].decode('utf-8')
+        return base64.b64encode(uuid.uuid4().bytes)[:6].decode("utf-8")
 
-    def _memoize_version(self, f, args=None, reset=False, delete=False,
-                         timeout=None, forced_update=False):
+    def _memoize_version(
+        self,
+        f,
+        args=None,
+        reset=False,
+        delete=False,
+        timeout=None,
+        forced_update=False,
+    ):
         """Updates the hash version associated with a memoized function or
         method.
         """
@@ -481,17 +524,23 @@ class Cache(object):
             dirty = True
 
         if dirty:
-            self.cache.set_many(dict(zip(fetch_keys, version_data_list)),
-                                timeout=timeout)
+            self.cache.set_many(
+                dict(zip(fetch_keys, version_data_list)), timeout=timeout
+            )
 
-        return fname, ''.join(version_data_list)
+        return fname, "".join(version_data_list)
 
-    def _memoize_make_cache_key(self, make_name=None, timeout=None,
-                                forced_update=False):
+    def _memoize_make_cache_key(
+        self,
+        make_name=None,
+        timeout=None,
+        forced_update=False,
+        hash_method=hashlib.md5,
+    ):
         """Function used to create the cache_key for memoized functions."""
 
         def make_cache_key(f, *args, **kwargs):
-            _timeout = getattr(timeout, 'cache_timeout', timeout)
+            _timeout = getattr(timeout, "cache_timeout", timeout)
             fname, version_data = self._memoize_version(
                 f, args=args, timeout=_timeout, forced_update=forced_update
             )
@@ -509,13 +558,14 @@ class Cache(object):
 
             updated = u"{0}{1}{2}".format(altfname, keyargs, keykwargs)
 
-            cache_key = hashlib.md5()
-            cache_key.update(updated.encode('utf-8'))
+            cache_key = hash_method()
+            cache_key.update(updated.encode("utf-8"))
             cache_key = base64.b64encode(cache_key.digest())[:16]
-            cache_key = cache_key.decode('utf-8')
+            cache_key = cache_key.decode("utf-8")
             cache_key += version_data
 
             return cache_key
+
         return make_cache_key
 
     def _memoize_kwargs_to_args(self, f, *args, **kwargs):
@@ -534,7 +584,7 @@ class Cache(object):
 
         for i in range(args_len):
             arg_default = get_arg_default(f, i)
-            if i == 0 and arg_names[i] in ('self', 'cls'):
+            if i == 0 and arg_names[i] in ("self", "cls"):
                 #: use the repr of the class instance
                 #: this supports instance methods for
                 #: the memoized functions, giving more
@@ -576,9 +626,16 @@ class Cache(object):
             new_args.append(arg)
 
         new_args.extend(args[len(arg_names):])
-        return tuple(new_args), OrderedDict(sorted(
-            (k, v) for k, v in iteritems(kwargs) if k in kw_keys_remaining
-        ))
+        return (
+            tuple(new_args),
+            OrderedDict(
+                sorted(
+                    (k, v)
+                    for k, v in iteritems(kwargs)
+                    if k in kw_keys_remaining
+                )
+            ),
+        )
 
     def _bypass_cache(self, unless, f, *args, **kwargs):
         """Determines whether or not to bypass the cache by calling unless().
@@ -588,7 +645,11 @@ class Cache(object):
         bypass_cache = False
 
         if callable(unless):
-            argspec = inspect.getargspec(unless)
+            try:
+                # Python >= 3.0
+                argspec = inspect.getfullargspec(unless)
+            except AttributeError:
+                argspec = inspect.getargspec(unless)
 
             # If unless() takes args, pass them in.
             if len(argspec.args) > 0 and argspec.varargs and argspec.keywords:
@@ -599,8 +660,14 @@ class Cache(object):
 
         return bypass_cache
 
-    def memoize(self, timeout=None, make_name=None, unless=None,
-                forced_update=None):
+    def memoize(
+        self,
+        timeout=None,
+        make_name=None,
+        unless=None,
+        forced_update=None,
+        hash_method=hashlib.md5,
+    ):
         """Use this to cache the result of a function, taking its arguments
         into account in the cache key.
 
@@ -655,10 +722,13 @@ class Cache(object):
                               cache value will be updated regardless cache
                               is expired or not. Useful for background
                               renewal of cached functions.
+        :param hash_method: Default hashlib.md5. The hash method used to
+                            generate the keys for cached results.
 
         .. versionadded:: 0.5
             params ``make_name``, ``unless``
         """
+
         def memoize(f):
             @functools.wraps(f)
             def decorated_function(*args, **kwargs):
@@ -678,34 +748,39 @@ class Cache(object):
                 except Exception:
                     if self.app.debug:
                         raise
-                    logger.exception("Exception possibly due to "
-                                     "cache backend.")
+                    logger.exception(
+                        "Exception possibly due to " "cache backend."
+                    )
                     return f(*args, **kwargs)
 
                 if rv is None:
                     rv = f(*args, **kwargs)
                     try:
                         self.cache.set(
-                            cache_key, rv,
-                            timeout=decorated_function.cache_timeout
+                            cache_key,
+                            rv,
+                            timeout=decorated_function.cache_timeout,
                         )
                     except Exception:
                         if self.app.debug:
                             raise
-                        logger.exception("Exception possibly due to "
-                                         "cache backend.")
+                        logger.exception(
+                            "Exception possibly due to " "cache backend."
+                        )
                 return rv
 
             decorated_function.uncached = f
             decorated_function.cache_timeout = timeout
             decorated_function.make_cache_key = self._memoize_make_cache_key(
-                make_name=make_name, timeout=decorated_function,
-                forced_update=forced_update
+                make_name=make_name,
+                timeout=decorated_function,
+                forced_update=forced_update,
+                hash_method=hash_method,
             )
-            decorated_function.delete_memoized = \
-                lambda: self.delete_memoized(f)
+            decorated_function.delete_memoized = lambda: self.delete_memoized(f)
 
             return decorated_function
+
         return memoize
 
     def delete_memoized(self, f, *args, **kwargs):
@@ -730,7 +805,7 @@ class Cache(object):
             43
             >>> random_func()
             43
-            >>> cache.delete_memoized('random_func')
+            >>> cache.delete_memoized(random_func)
             >>> random_func()
             16
             >>> param_func(1, 2)
@@ -739,7 +814,7 @@ class Cache(object):
             32
             >>> param_func(2, 2)
             47
-            >>> cache.delete_memoized('param_func', 1, 2)
+            >>> cache.delete_memoized(param_func, 1, 2)
             >>> param_func(1, 2)
             13
             >>> param_func(2, 2)
@@ -780,8 +855,7 @@ class Cache(object):
             >>> adder2.add(3)
             3.72341788
 
-        :param fname: Name of the memoized function, or a reference to
-                      the function.
+        :param fname: The memoized function.
         :param \*args: A list of positional parameters used with
                        memoized function.
         :param \**kwargs: A dict of named parameters used with
@@ -789,20 +863,20 @@ class Cache(object):
 
         .. note::
 
-            Flask-Cache uses inspect to order kwargs into positional args when
+            Flask-Caching uses inspect to order kwargs into positional args when
             the function is memoized. If you pass a function reference into
-            ``fname`` instead of the function name, Flask-Cache will be able
-            to place the args/kwargs in the proper order, and delete the
-            positional cache.
+            ``fname``, Flask-Caching will be able to place the args/kwargs in
+            the proper order, and delete the positional cache.
 
             However, if ``delete_memoized`` is just called with the name of the
             function, be sure to pass in potential arguments in the same order
-            as defined in your function as args only, otherwise Flask-Cache
-            will not be able to compute the same cache key.
+            as defined in your function as args only, otherwise Flask-Caching
+            will not be able to compute the same cache key and delete all
+            memoized versions of it.
 
         .. note::
 
-            Flask-Cache maintains an internal random version hash for
+            Flask-Caching maintains an internal random version hash for
             the function. Using delete_memoized will only swap out
             the version hash, causing the memoize function to recompute
             results and put them into another key.
@@ -816,9 +890,11 @@ class Cache(object):
             backend.
         """
         if not callable(f):
-            raise DeprecationWarning("Deleting messages by relative name is "
-                                     "no longer reliable, please switch to a "
-                                     "function reference.")
+            raise DeprecationWarning(
+                "Deleting messages by relative name is "
+                "no longer reliable, please switch to a "
+                "function reference."
+            )
 
         if not (args or kwargs):
             self._memoize_version(f, reset=True)
@@ -838,8 +914,10 @@ class Cache(object):
             in the cache backend.
         """
         if not callable(f):
-            raise DeprecationWarning("Deleting messages by relative name is "
-                                     "no longer reliable, please use a "
-                                     "function reference.")
+            raise DeprecationWarning(
+                "Deleting messages by relative name is "
+                "no longer reliable, please use a "
+                "function reference."
+            )
 
         self._memoize_version(f, delete=True)
